// ///////////////////////////////////////////////////////////////////
//
//  File:      astro_image.cxx
//
//  Version:   1.0
//
//  Author:    Reiner Rohlfs
//
//  History:
//
// ///////////////////////////////////////////////////////////////////
#include <TROOT.h>
#include <TFile.h>
#include <TApplication.h>
#include <TCanvas.h>

#include "pil.h"
#include "param_api.h"

#include "astroImage.h"


VoidFuncPtr_t initFuncs[] = {0};
TROOT root ("ISDC", "osm_hk_display", initFuncs);

//_____________________________________________________________________________
int GetParams(int argc, char ** argv, TApplication * app,
              char * fileName, UInt_t * extension, char * imageName)
//  reads all program parameters either from the command line , form a GUI 
//  or from the parameter file. PIL is used.
{

   int status = ISDC_OK;

   
   for (int par = 1; par < argc; par++)
      if (strcmp("--g",   argv[par]) == 0 ||
          strcmp("--gui", argv[par]) == 0    )
         {
         // use the GUI to ask for the parameters
         ParamGui gui(argc, argv, app);
         if (gui.Quit())        exit(0);
         if (gui.Error() != ISDC_OK)   return gui.Error();
      
         strcpy(fileName,       gui.GetString("filename"));
         strcpy(imageName,      gui.GetString("imagename"));
         *extension =  gui.GetInt("extension");

         return gui.Error();
         }

   do
      {
      status = PILInit(argc, argv);
      if (status != ISDC_OK) break;

      status = PILGetFname("filename", fileName);
      if (status != ISDC_OK) break;

      int readExtension;
      status = PILGetInt("extension", &readExtension);
      if (status != ISDC_OK) break;
      *extension = readExtension;

      status = PILGetString("imagename", imageName);
      if (status != ISDC_OK) break;

      } while (0);

   PILClose(ISDC_OK);

   return status;
}

//_____________________________________________________________________________
int main(int argc, char ** argv)
{
   char     fileName[512];
   UInt_t   extension;
   char     imageName[40];

 	TApplication theApp("imageDisplay", &argc, argv);

   // get the program parameters
   if (int status = GetParams(argc, argv, &theApp, fileName, &extension, imageName))
      {
      printf("Error during reading parameters: %d\n", status);
      return 0;
      }

   // open a window
   TCanvas * win = new TCanvas("name", "title", 20, 20, 800, 750);
   win->ToggleEventStatus();
   win->Draw();

   // read the image from a file
   AstroImage * img;
   img = new AstroImage(fileName, imageName, extension);

   // draw the image and set the title of the window to the image name
   img->Draw();
   win->SetTitle(img->GetName());
   
   TFError::PrintErrors();

   theApp.Run();
}
