###############################################################################
#
#   File:      astroroot_init_env.csh
#   Version:   1.0
#   Component: astroroot_init_env
#
#   Author(s): Mathias.Beck@obs.unige.ch (MB)  (isdc_init_env.csh)
#	       Reiner.Rohlfs   version 1.0 astroroot_init_env.csh
#
#   Purpose:   setup wrapper for C-shell flavours
#
#   Revision History:
#
#      Version 1.0:  modified from isdc_init_env.csh version 0.4.3
#
###############################################################################

set my_status=0
set my_warning_base=40
set my_error_base=1

#
# WORK_ENV must be set prior to calling this script
#
if ($?WORK_ENV == 0) then
   set my_status = 1
else
   set ASRO_SETUP_SCRIPT = ${WORK_ENV}/bin/astroroot_setup_env.sh

   if (-x ${ASRO_SETUP_SCRIPT}) then
      if ($1 == "--h" || $1 == "--help") then
      	 ${ASRO_SETUP_SCRIPT} --help
      	 set my_status = ${status}
      else if ($1 == "--v" || $1 == "--version") then
      	 ${ASRO_SETUP_SCRIPT} --version
      	 set my_status = ${status}
      else
      	 set astroroot_init_env = `${ASRO_SETUP_SCRIPT} csh`
      	 set my_status = ${status}
      	 test ${my_status} -lt ${my_warning_base} -a "x${astroroot_init_env}" != "x"
      	 if ($status == 0) then
      	    if (-f "${astroroot_init_env}") then
      	       source ${astroroot_init_env}
      	       rm -f ${astroroot_init_env}
	    endif
      	 endif
      	 unset astroroot_init_env
      endif
   else
      set my_status = 7
   endif
  
   unset ASRO_SETUP_SCRIPT
endif

if (${my_status} != 0) then
   switch (${my_status})
      case 1:
      	 echo 'Error  : code -35751: env. variable $WORK_ENV is not set'
      	 breaksw
      case 3:
      	 echo 'Error  : code -35753: directory $HOME/pfiles cannot be created'
      	 breaksw
      case 4:
      	 echo 'Error  : code -35754: $WORK_ENV/help/astroroot_init_env.txt not found'
      	 breaksw
      case 5:
      	 echo 'Error  : code -35755: Internal error: wrong number of arguments'
      	 breaksw
      case 6:
      	 echo 'Error  : code -35756: Internal error: unsupported shell flavour'
      	 breaksw
      case 7:
      	 echo 'Error  : code -35757: Internal error: could not execute $WORK_ENV/bin/astroroot_setup_env.sh'
      	 breaksw
      case 42:
      	 echo 'Warning: code -35792: env. variable $ROOTSYS is not set'
      	 breaksw
   endsw
endif

exit ${my_status}
