###############################################################################
#
#   File:      astroroot_init_env.sh
#   Version:   1.0
#   Component: astroroot_init_env
#
#   Author(s): Mathias.Beck@obs.unige.ch (MB)  (isdc_init_env.csh)
#	       Reiner.Rohlfs   version 1.0 astroroot_init_env.csh
#
#   Purpose:   setup wrapper for C-shell flavours
#
#   Revision History:
#
#      Version 1.0:  modified from isdc_init_env.csh version 0.4.3
#
###############################################################################

my_status=0
my_warning_base=40
my_error_base=1

#
# ISDC_ENV must be set prior to calling this script
#
if [ x${ISDC_ENV} = x ] ; then
   my_status=1
else
   ASRO_SETUP_SCRIPT=${ISDC_ENV}/bin/astroroot_setup_env.sh

   if [ -x ${ASRO_SETUP_SCRIPT} ] ; then
      if [ x$1 = 'x--h' -o  x$1 = 'x--help' ] ; then
      	 ${ASRO_SETUP_SCRIPT} --help
      	 my_status=$?
      elif [ x$1 = 'x--v' -o x$1 = 'x--version' ] ; then
      	 ${ASRO_SETUP_SCRIPT} --version
      	 my_status=$?
      else
      	 astroroot_init_env=`${ASRO_SETUP_SCRIPT} sh`
      	 my_status=$?
      	 #
	 # SPR-03374
	 #
      	 test ${my_status} -lt ${my_warning_base} -a "x${astroroot_init_env}" != x
      	 if [ $? -eq 0 ] ; then
      	    if [ -f "${astroroot_init_env}" ]; then
      	       . ${astroroot_init_env}
      	       rm -f ${astroroot_init_env}
      	    fi
      	 fi
      	 unset astroroot_init_env
      fi
   else
      my_status=7
   fi

   unset ASRO_SETUP_SCRIPT
fi

if [ ${my_status} -ne 0 ] ; then
   case ${my_status} in
      1) echo 'Error   : code -35751: env. variable $ISDC_ENV is not set'
         ;;
      3) echo 'Error   : code -35753: directory $HOME/pfiles cannot be created'
         ;;
      4) echo 'Error   : code -35754: $ISDC_ENV/help/astroroot_init_env.txt not found'
         ;;
      5) echo 'Error   : code -35755: Internal error: wrong number of arguments'
         ;;
      6) echo 'Error   : code -35756: Internal error: unsupported shell flavour'
         ;;
      7) echo 'Error   : code -35757: Internal error: could not execute $ISDC_ENV/bin/astroroot_setup_env.sh'
         ;;
      42) echo 'Warning: code -35792: env. variable $ROOTSYS is not set'
         ;;
   esac
fi

