/***********************************************************************/
/*                                                                     */
/*                    INTEGRAL SCIENCE DATA CENTRE                     */
/*                                                                     */
/*                      PARAMETER - GUI LIBRARY                        */
/*                                                                     */
/*                                                                     */
/*  File:      tools.cpp                                               */
/*  Version:   1.0                                                     */
/*  Component: param_gui_lib                                           */
/*                                                                     */
/*  Authors:   Reiner Rohlfs (RR)                                      */
/*             ISDC                                                    */
/*                                                                     */
/*  Purpose:   Some tools are defined here.                            */
/*                                                                     */
/*  Revision History:                                                  */
/*                                                                     */
/*  version 1.0: 14/08/02 RR  First version                            */
/*                                                                     */
/***********************************************************************/
#include <stdio.h>
#include <sys/stat.h>
#include <sys/types.h>
#include <unistd.h>
#include <fcntl.h>


#include <TGButton.h>
#include <TRootHelpDialog.h>
#include <TGMsgBox.h>
#include <TGXYLayout.h>

#include "pil.h"

#include "param_gui.h"
#include "tools.h"
#include "widget.h"

using namespace std;


//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////

/*****************************************************************************/
/*  Function :     StdButton::StdButton                                      */
/*                                                                           */
/*  Description:   Class to create the standard buttons                      */
/*                                                                           */
/*  Return code:   None                                                      */
/*                                                                           */
/*  Parameter arguments:                                                     */
/*                                                                           */
/*  name        I/O     valid range      description                         */
/*                                                                           */
/*  type        input   0, 1             0: four or fife buttons will be     */
/*                                          created                          */
/*                                       1: Ok and Help button will be       */
/*                                          created                          */
/*                                                                           */
/*****************************************************************************/
StdButton::StdButton(Int_t type)
{
   m_btWidth = 8;
   m_yOff    = 2;

   m_type    = type;
   m_numButtons = (type == 0) ? 6 : 2;
}
/*****************************************************************************/
/*  Function :     StdButton::Create                                         */
/*                                                                           */
/*  Description:   Create the button in a window                             */
/*                                                                           */
/*  Return code:   None                                                      */
/*                                                                           */
/*  Parameter arguments:                                                     */
/*                                                                           */
/*  name        I/O     valid range      description                         */
/*                                                                           */
/*  window      input                    the buttons are created on this     */
/*                                       window                              */
/*  winSize     input                    defines the size of window. The     */
/*                                       buttons are placed at the upper     */
/*                                       right corner                        */
/*                                                                           */
/*****************************************************************************/
void StdButton::Create(TGMainFrame * window, WinSize * winSize)
{
   TGTextButton * button;

   Double_t xPos = winSize->GetWidth() - 1 - m_btWidth;

   Double_t yPos = 0;

   if (m_type == 0)
      {
      button = new TGTextButton(window, "&Save", BT_SAVE);
      window->AddFrame(button, 
            new TGXYLayoutHints(xPos, 1 + yPos * m_yOff,
                                 m_btWidth, m_yOff - 0.2));  
      yPos++;

      button = new TGTextButton(window, "&Save As", BT_SAVE_AS);
      window->AddFrame(button, 
            new TGXYLayoutHints(xPos, 1 + yPos * m_yOff,
                                 m_btWidth, m_yOff - 0.2));  
      yPos++;

      button = new TGTextButton(window, "&Run", BT_RUN);
      window->AddFrame(button, 
            new TGXYLayoutHints(xPos, 1 + yPos * m_yOff, 
                                 m_btWidth, m_yOff - 0.2));  
      yPos++;

      button = new TGTextButton(window, "&Quit", BT_QUIT);
      window->AddFrame(button, 
            new TGXYLayoutHints(xPos, 1 + yPos * m_yOff, 
                                 m_btWidth, m_yOff - 0.2));  
      yPos++;
      }
   else
      {
      button = new TGTextButton(window, "&Ok", BT_OK);
      window->AddFrame(button, 
            new TGXYLayoutHints(xPos, 1 + yPos * m_yOff, 
                                 m_btWidth, m_yOff - 0.2));  
      yPos++;
      }

   button = new TGTextButton(window, "&Help", BT_HELP);
   window->AddFrame(button, 
         new TGXYLayoutHints(xPos, 1 + yPos * m_yOff, 
                                 m_btWidth, m_yOff - 0.2));  

}

//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////

/*****************************************************************************/
/*  Function :     WinSize::WinSize                                          */
/*                                                                           */
/*  Description:   Class to determine the size of a window depending on the  */
/*                 size and the position of the widgets in a given window    */
/*                                                                           */
/*  Return code:   None                                                      */
/*                                                                           */
/*  Parameter arguments:                                                     */
/*                                                                           */
/*  name        I/O     valid range      description                         */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/
WinSize::WinSize()
{
   m_width     = 0;
   m_height    = 0;
   m_stdWidth  = 0;
   m_stdHeight = 0;
   m_mode      = 0;
}
/*****************************************************************************/
/*  Function :     WinSize::WinSize                                          */
/*                                                                           */
/*  Description:   Constructor                                               */
/*                                                                           */
/*  Return code:   None                                                      */
/*                                                                           */
/*  Parameter arguments:                                                     */
/*                                                                           */
/*  name        I/O     valid range      description                         */
/*                                                                           */
/*  stdButtons  input                    class of the standard buttons       */
/*                                       these define a minimum size of the  */
/*                                       window                              */
/*                                                                           */
/*****************************************************************************/
WinSize::WinSize(StdButton & stdButton)
{
   m_width  = m_stdWidth  = stdButton.GetWidth();
   m_height = 0;
   m_stdHeight = stdButton.GetHeight();

   m_mode = 0;
}
/*****************************************************************************/
/*  Function :     osm_gti                                                   */
/*                                                                           */
/*  Description:                                                             */
/*                                                                           */
/*  Return code:   None                                                      */
/*                                                                           */
/*  Parameter arguments:                                                     */
/*                                                                           */
/*  name        I/O     valid range      description                         */
/*                                                                           */
/*                                                                           */
/*****************************************************************************/
void WinSize::Or(WinSize & winSize)
{
   if (winSize.m_width  > m_width ) m_width  = winSize.m_width;
   if (winSize.m_height > m_height) m_height = winSize.m_height;
}
/*****************************************************************************/
/*  Function :     WinSize::SetSize                                          */
/*                                                                           */
/*  Description:   Redefine the size depending on the new widget "widget"    */
/*                                                                           */
/*  Return code:   None                                                      */
/*                                                                           */
/*  Parameter arguments:                                                     */
/*                                                                           */
/*  name        I/O     valid range      description                         */
/*                                                                           */
/*  widget      input                    this widget should be placed on     */
/*                                       on a window, but only if its z      */
/*                                       koordinate == input parameter z     */
/*  z          input                     Z koordinate name of the window     */
/*                                       for which this WinSize determines   */
/*                                       the size.                           */
/*                                                                           */
/*****************************************************************************/
void WinSize::SetSize (Widget * widget, const char * z)
{
   if (widget->GetZ() != z)
      return;

   if (m_mode == 0 && !widget->SizeDefined())
      widget->MakeSize();


   if (m_mode == 1 && widget->PosDefined())
      // there is nothing to do here
      return;

   if (m_mode == 1)
      {
      // we have to define the position
      widget->SetXPos(0);
      widget->SetYPos(m_height);
      }

   if (widget->PosDefined())
      {
      double w_y = widget->GetYPos() + widget->GetHeight();
      if (w_y > m_height)
      m_height = w_y;

      double w_x = widget->GetXPos() + widget->GetWidth();
      if (widget->GetYPos() > m_stdHeight)
         {
         if (w_x > m_width)
            m_width = w_x;
         }
      else
         {
         if (w_x > m_width - m_stdWidth)
            m_width = w_x + m_stdWidth;
         }
      }
}

//////////////////////////////////////////////////////////////
//////////////////////////////////////////////////////////////

/*****************************************************************************/
/*  Function :     SubStr::SubStr                                            */
/*                                                                           */
/*  Description:   Class to cut a long string into substrings                */
/*                 the substrings are seperated by |                         */
/*                                                                           */
/*  Return code:   None                                                      */
/*                                                                           */
/*  Parameter arguments:                                                     */
/*                                                                           */
/*  name        I/O     valid range      description                         */
/*                                                                           */
/*  str       input                      this string will be cut into        */
/*                                       substrings                          */
/*                                                                           */
/*****************************************************************************/
SubStr::SubStr(const string & str)
  : m_str(str)
{
   m_index = 0;
}
/*****************************************************************************/
/*  Function :     SubStr::SubStr                                            */
/*                                                                           */
/*  Description:   Class to cut a long string into substrings                */
/*                 the substrings are seperated by |                         */
/*                                                                           */
/*  Return code:   None                                                      */
/*                                                                           */
/*  Parameter arguments:                                                     */
/*                                                                           */
/*  name        I/O     valid range      description                         */
/*                                                                           */
/*  str       input                      this string will be cut into        */
/*                                       substrings                          */
/*                                                                           */
/*****************************************************************************/
SubStr::SubStr(const char * str)
  : m_str(str)
{
   m_index = 0;
}
/*****************************************************************************/
/*  Function :     SubStr::Next                                              */
/*                                                                           */
/*  Description:   The next substring is returned in subStr                  */
/*                                                                           */
/*  Return code:   true if there is an further substring, else false         */
/*                                                                           */
/*  Parameter arguments:                                                     */
/*                                                                           */
/*  name        I/O     valid range      description                         */
/*                                                                           */
/*  subStr      output                   the next sub string                 */
/*                                                                           */
/*****************************************************************************/
bool SubStr::Next(string & subStr)
{
   if (m_index > m_str.size())
      // we have already found all substrings
      return false;

   string newStr;
   int newIndex = m_str.find('|', m_index);
   if (newIndex != m_str.npos)
      {
      newStr = m_str.substr(m_index, newIndex - m_index);
      m_index = newIndex + 1;
      }
   else
      {
      // the last string after the last |
      newStr = m_str.substr(m_index, m_str.size() - m_index);
      m_index = m_str.size() + 1;
      }

   // remove blanks and tabs at the front and the end of the new subStr
   StripBlanks(newStr);

   subStr = newStr;
   return true;
}
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

/*****************************************************************************/
/*  Function :     StripBlanks                                               */
/*                                                                           */
/*  Description:   Removes all blanks and tabs at the beginning and the      */
/*                 end of the str                                            */
/*                                                                           */
/*  Return code:   None                                                      */
/*                                                                           */
/*  Parameter arguments:                                                     */
/*                                                                           */
/*  name        I/O     valid range      description                         */
/*                                                                           */
/*  str         input                    blanks and tabs at the beginning    */
/*              output                   and at the end of this string will  */
/*                                       be removed. The modified string     */
/*                                       is returned                         */
/*                                                                           */
/*****************************************************************************/
void StripBlanks(string & str)
{
   int num = 0;
   while (num < str.size() && 
      (str.at(num) == ' ' || str.at(num) == '\t'))
      num++;
   if (num)
      str.erase(0, num);
   
   num = str.size() - 1;
   while (num >= 0 && 
         (str.at(num) == ' ' || str.at(num) == '\t'))
      num--;
   if (num < str.size() -1)
      str.erase(num + 1, str.size() - 1 + num);
}

////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////

/*****************************************************************************/
/*  Function :     OpenHelp                                                  */
/*                                                                           */
/*  Description:   Opens a window and displays the help text of a program    */
/*                                                                           */
/*  Return code:   None                                                      */
/*                                                                           */
/*  Parameter arguments:                                                     */
/*                                                                           */
/*  name        I/O     valid range      description                         */
/*                                                                           */
/*  parentWin   input                    the parent window                   */
/*  filename    input                    name of the parameter file          */
/*                                       This filename is used to find the   */
/*                                       help text file.                     */
/*                                                                           */
/*****************************************************************************/
void OpenHelp(const TGWindow * parentWin, const char * filename)
{
   char f1[512];
   strcpy(f1, filename);

   // replace .par by .txt
   char * pos = strrchr(f1, '.');
   if (pos) *pos = 0;
   strcat(f1, ".txt");

   // remove the path and set the help directory instead
   pos = strrchr(f1, '/');
   if (pos == NULL)
      pos = f1;
   char * env = getenv("WORK_ENV");
   if (env == NULL) 
      {
      // try with old ISDC_ENV
      env = getenv("ISDC_ENV");
      if (env == NULL) 
         {
         new TGMsgBox(gClient->GetRoot(), parentWin, 
                     "Error Message", 
                     "Environment variable WORK_ENV is not defined.\n"
                     "Cannot find the help file!",
   	    kMBIconExclamation, kMBOk);
         return;
         }
      }
   char f2[512];
   sprintf(f2, "%s/help/%s", env, pos);

   // get size of file
   struct stat buf;
   if (stat(f2, &buf) == -1)
      {
      char msg[612];
      sprintf(msg, "File %s is missing\nCannot display help text.",
            f2);
      new TGMsgBox(gClient->GetRoot(), parentWin, 
                  "Error Message", msg,
	    kMBIconExclamation, kMBOk);
      return;
      }

   char * buffer = new char[buf.st_size];
   int file = open(f2, O_RDONLY);
   if (file == -1)
      {
      char msg[612];
      sprintf(msg, "Cannot open file\n%s\nCannot display help text",
            f2);
      new TGMsgBox(gClient->GetRoot(), parentWin, 
                  "Error Message", msg,
	    kMBIconExclamation, kMBOk);
      return;
      }

   read(file, buffer, buf.st_size);
   close(file);

   TRootHelpDialog * win = new TRootHelpDialog(
               parentWin, filename, 600,450);

   win->SetText(buffer);
   delete [] buffer;
   win->Popup();

//   gClient->WaitFor(win);

}
////////////////////////////////////////////////////////////////////////
////////////////////////////////////////////////////////////////////////


/*****************************************************************************/
/*  Function :     find                                                      */
/*                                                                           */
/*  Description:   Finds the widget with name "name" in the list of all      */
/*                 widgets and returns an iterator to it                     */
/*                                                                           */
/*                                                                           */
/*  Return code:   iterator to the found widget or widget.end() if a         */
/*                 widget with name "name" does not exist                    */
/*                                                                           */
/*  Parameter arguments:                                                     */
/*                                                                           */
/*  name        I/O     valid range      description                         */
/*                                                                           */
/*  widgets     input                    list of all widgets                 */
/*  name        input                    a widget with this name is looked   */
/*                                       for                                 */
/*                                                                           */
/*****************************************************************************/
static I_Widgets find(T_Widgets & widgets, string & name)
{
  I_Widgets i_w = widgets.begin();
  while( i_w != widgets.end())
    {
    if ((*i_w)->GetName() == name)
      break;
    i_w++;
    }
  return i_w;
}

/*****************************************************************************/
/*  Function :     ParseParamLine                                            */
/*                                                                           */
/*  Description:   Parses one parameter line to create a new widget          */
/*                                                                           */
/*  Return code:   None                                                      */
/*                                                                           */
/*  Parameter arguments:                                                     */
/*                                                                           */
/*  name        I/O     valid range      description                         */
/*                                                                           */
/*  line        input                    one line of the parameter file      */
/*  widget      output                   list of all widgets                 */
/*                                                                           */
/*****************************************************************************/
static void ParseParamLine(char * line, T_Widgets & widgets)
{
   char *ptab[7];
   int r, tab;

   r = PIL_OK;
   for (tab = 0; tab < 7; tab++)
      { 
      // skipp tabs and blanks
      for (;; line++) 
         if ((' ' != *line) && ('\t' != *line)) 
            break;

      switch (*line)
         { 
         case '\'': ;
         case '"' : 
            ptab[tab] = ++line;	/* found string in "xxxx" or 'xxxx' */
            r = PIL_mark_eos(&line, line[-1]);
            if (PIL_OK == r)
               { 
               for (;; line++) 
                  if ((' ' != *line) && ('\t' != *line)) 
                     break;
               switch (*line)
                  { 
                  case ',' : 
                     line++; break;
                  case '\n': ;
                  case '\r': ;
                  case 0   : 
                     *line = 0;
                     r = PIL_EOS;
                     break;
                  }
               }
            break;	
         
         case '\n': ;
         case '\r': ;
         case 0   : 
            *line = 0;		/* found EOS */
            r = PIL_EOS;
            break;
        default  : 
            ptab[tab] = line;	// found regular string (from ',' to ',')
            r = PIL_mark_eos(&line, ',');
            break;
         }

      if (tab >= 6) if (PIL_EOS == r) r = PIL_OK;
      if (PIL_OK != r) break;
      }
   
   if (7 != tab) return;

   Widget * widget;

   if (strchr(ptab[4], '|') && ptab[1][0] != 'b')
      widget = new W_ComboBox(ptab[0], ptab[1][0]);
   else
      {
      switch (ptab[1][0])
         {
         case 'b':
            widget = new W_CheckButton(ptab[0]);
            break;
         case 'i':
            widget = new W_IntEntry(ptab[0]);
            break;
         case 'r':
            widget = new W_RealEntry(ptab[0]);
            break;
         case 's':
            widget = new W_TextEntry(ptab[0], T_STRING);
            break;
         case 'f':
            widget = new W_TextEntry(ptab[0], T_FILE);
            break;
         default:
            return;
         }
      }   

   widget->SetParamType(ptab[1]);
   widget->SetMode(ptab[2]);
   widget->SetValue(ptab[3]);
   widget->SetMin(ptab[4]);
   widget->SetMax(ptab[5]);
   widget->SetPrompt(ptab[6]);

   widgets.push_back(widget);
}
/*****************************************************************************/
/*  Function :     ParseGuiLine                                              */
/*                                                                           */
/*  Description:   Parse one line of the parameter file to add some          */
/*                 attributes to a widget or creagte a new group widget      */
/*                                                                           */
/*  Return code:   None                                                      */
/*                                                                           */
/*  Parameter arguments:                                                     */
/*                                                                           */
/*  name        I/O     valid range      description                         */
/*                                                                           */
/*  line        input                    one line of the parameter file      */
/*  widget      output                   list of all widgets                 */
/*                                                                           */
/*****************************************************************************/
static void ParseGuiLine(char * line, T_Widgets & widgets)
{
   string file_line = line;

   // skipp # GUI
   line += 5;

   // skipp tabs and blanks
   while(*line == ' ' || *line == '\t')
      line++;

   // find name of parameter  
   char * end = strpbrk(line, ",");
   if (end == NULL)  return;

   *end = 0;

   // skipp tabs and blanks at the end 
   char * tmp = end -1;
   while(*tmp == ' ' || *tmp =='\t')
      { *tmp = 0; tmp--; }

   string name(line);

   I_Widgets i_w;
   Widget * widget;

   // try to find the widget 
   if ((i_w = find(widgets, name)) == widgets.end())
      {
      // thers is no widget for this parameter

      if (strncmp(line, "_FRAME_", 7) == 0)
         {
         // a new frame
         widget = new W_Frame(line);
         widgets.push_back(widget);
         }

      else if (strncmp(line, "_WINDOW_", 8) == 0)
         {
         // a new window
         widget = new W_Window(line);
         widgets.push_back( widget);
         }

      else if (strncmp(line, "_FOLDER_", 8) == 0)
         {
         // a new window
         widget = new W_Folder(line);
         widgets.push_back(widget);
         }

      else if (strncmp(line, "_COMMON_", 8) == 0 )
         {
         widget = NULL;
         }

      else
      // we do not know what to do wiht this line
         return;
      }
   else
      widget = *i_w;   

   while(end)
      {
      line = end+1;

      // skipp tabs and blanks
      while(*line == ' ' || *line == '\t')
         line++;

      if (*line == 0)
         break;

      // find next attribute  
      end = strpbrk(line, ",");
      if (end != NULL) 
         *end = 0;
      
      // skipp tabs and blanks at the end
      tmp = line + strlen(line) - 1;
      while(*tmp == ' ' || *tmp =='\t')
         { *tmp = 0; tmp--; }

      if (widget)
         widget->SetAttribute(line);
      else
         Widget::SetCommonAttribute(line);
      }
}
/*****************************************************************************/
/*  Function :     ParseLine                                                 */
/*                                                                           */
/*  Description:   Exported function to parse one line and create a new      */
/*                 widget or set some attributes                             */
/*                                                                           */
/*  Return code:   None                                                      */
/*                                                                           */
/*  Parameter arguments:                                                     */
/*                                                                           */
/*  name        I/O     valid range      description                         */
/*                                                                           */
/*  line        input                    one line of the parameter file      */
/*  widget      output                   list of all widgets                 */
/*                                                                           */
/*****************************************************************************/
void ParseLine(const char * line, T_Widgets & widgets)
{
   char tmpLine[PIL_LINESIZE];
   strcpy(tmpLine, line);

   if (strncmp(line, "# GUI", 5) == 0)
      ParseGuiLine(tmpLine, widgets);

   else if (line[0] != '#')
      ParseParamLine(tmpLine, widgets);

}


