/***********************************************************************/
/*                                                                     */
/*                    INTEGRAL SCIENCE DATA CENTRE                     */
/*                                                                     */
/*                      PARAMETER - GUI LIBRARY                        */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/*  File:      tools.h                                                 */
/*  Version:   1.0                                                     */
/*  Developer: Reiner Rohlfs (RR)                                      */
/*             ISDC                                                    */
/*                                                                     */
/*  Revision History:                                                  */
/*                                                                     */
/*  version 1.0: 14/08/02 RR  First version                            */
/*                                                                     */
/***********************************************************************/
#ifndef TOOLS_INCLUDED
#define TOOLS_INCLUDED
#include <string>

#include <Rtypes.h>

class WinSize;
class TGMainFrame;
class Widget;
class TGWindow;

#define BT_SAVE     0
#define BT_SAVE_AS  1
#define BT_RUN      2
#define BT_QUIT     3
#define BT_HELP     4
#define BT_OK       5


/////////////////////////////////////////////////////////////////////

class StdButton
{
public:
  StdButton(Int_t type);

  Double_t   GetWidth()   {return m_btWidth + 2;}
  Double_t   GetHeight()  {return m_yOff * m_numButtons + 2;}

  void Create(TGMainFrame * window, WinSize * winSize);

private:

  Double_t  m_btWidth;
  Double_t  m_yOff; 

  Int_t     m_type;
  Int_t     m_numButtons;
  
};

////////////////////////////////////////////////////////////////////

class WinSize
{
public:
  WinSize();
  WinSize(StdButton & stdButton);

  void Or(WinSize & winSize);

  void SetMode(Int_t mode) {m_mode = mode;}
  void SetSize (Widget * widget, const char * z);

  Double_t   GetWidth()   {return m_width;}
  Double_t   GetHeight()  
               {return (m_height > m_stdHeight) ? m_height : m_stdHeight;}

private:
  Double_t   m_width;
  Double_t   m_height;

  Double_t   m_stdWidth;
  Double_t   m_stdHeight;

  Int_t      m_mode;  // 0: get size from predefined widget sizes
                      // 1: set position of the other widgets and
                      //    than get the size from these widgets
  
};

///////////////////////////////////////////////////////////////////


class SubStr
{
public:
  SubStr(const std::string & str);
  SubStr(const char * str);

  bool Next(std::string & subStr);

private:
  std::string m_str;  // full input string
  int    m_index;     // index in m_str of start of  next substrin
};

// remove blanks and tabs at the frond and the end of str
extern void StripBlanks(std::string & str);

extern void OpenHelp(const TGWindow * window, const char * filename);

#endif // TOOLS_INCLUDED
