/***********************************************************************/
/*                                                                     */
/*                    INTEGRAL SCIENCE DATA CENTRE                     */
/*                                                                     */
/*                      PARAMETER - GUI LIBRARY                        */
/*                                                                     */
/*                                                                     */
/*                                                                     */
/*  File:      widget.h                                                */
/*  Version:   1.0                                                     */
/*  Developer: Reiner Rohlfs (RR)                                      */
/*             ISDC                                                    */
/*                                                                     */
/*  Revision History:                                                  */
/*                                                                     */
/*  version 1.0: 15/08/02 RR  First version                            */
/*                                                                     */
/***********************************************************************/
#ifndef WIDGET_INCLUDED
#define WIDGET_INCLUDED

#include <set>
#include <string>
#include <list>

#include "TGLabel.h"
#include "TGToolTip.h"
#include "TGTab.h"

#include "param_gui.h"

class TGCompositeFrame;
class TGTextEntry;
class TGTextBuffer;
class TGCheckButton;
class TGTextButton;
class TGNumberEntry;
class TGTextButton;
class TGComboBox;
class TGToolTip;
class TGXYLayoutHints;
class Widget;

// m_type types:
#define T_STRING   1
#define T_FILE     2
#define T_DOL      4
#define T_INT      8
#define T_REAL    16 

using namespace std;


/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

class ToolTipLabel : public TGLabel
{
public:
   ToolTipLabel(const TGCompositeFrame * win, const char * text);
   ~ToolTipLabel() {delete m_tip;}

   void SetToolTip(const char * tip);

   Bool_t HandleCrossing(Event_t *event);

private:
   TGToolTip * m_tip;
};

/////////////////////////////////////////////////////////////////////
/////////////////////////////////////////////////////////////////////

class SubWindow : public TGMainFrame
{
public:
   SubWindow(const string & name);
   ~SubWindow();

   Bool_t ProcessMessage(Long_t msg, Long_t param1, Long_t param2);
   virtual void CloseWindow();

private:

   string    m_name;
};

class W_Window : public Widget
{
public:
   W_Window(const char * name);

   virtual void Create(TGCompositeFrame * win, const char * winName);
   virtual void ProcessMessage(Long_t msg, Long_t param1, Long_t param2);

   virtual void CloseWidget(string & backGround);
private:
   TGTextButton * m_button;

   SubWindow * m_window;
};

/////////////////////////////////////////////////////////////////////

class FolderTab : public TGCompositeFrame
{
public:
   FolderTab(TGCompositeFrame * topWin, TGXYLayoutHints * hints,
            string & name);
   ~FolderTab();

   Bool_t ProcessMessage(Long_t msg, Long_t param1, Long_t param2);
};

class FolderWindow : public TGTab
{
public:
   FolderWindow(TGCompositeFrame * win, Double_t width, 
               Double_t height, string & tabNames);
   ~FolderWindow();

private:
   TGXYLayoutHints  * m_hints;
   list<FolderTab *> m_tabs;
};

class W_Folder : public Widget
{
public:
   W_Folder(const char * name);

   virtual void MakeSize();


   virtual void Create(TGCompositeFrame * win, const char * winName);
   virtual void BeforeOpen(const string & backGround);
   virtual void CloseWidget(string & backGround);
private:

};

/////////////////////////////////////////////////////////////////////

class W_Frame : public Widget
{
public:
   W_Frame(const char * name);

   virtual void CreateFrame(TGCompositeFrame * win, const char * winName);

private:
};

/////////////////////////////////////////////////////////////////////

class W_CheckButton : public Widget
{
public:
   W_CheckButton(const char * name);

   virtual void Init();
   virtual void Create(TGCompositeFrame * win, const char * winName);
   virtual void ProcessMessage(Long_t msg, Long_t param1, Long_t param2);
   virtual void ValueChanged(string & widgetName, string & value);
   virtual void UpdateParamFile();

private:
   TGCheckButton * m_button;
};

/////////////////////////////////////////////////////////////////////

class W_ComboBox : public Widget
{
public:
   W_ComboBox(const char * name, char type);

   virtual void SetMin      (const char * min);

   virtual void Init();
   virtual void Create(TGCompositeFrame * win, const char * winName);
   virtual void BeforeOpen(const string & backGround); 
   virtual void ValueChanged(string & widgetName, string & value);
   virtual void ProcessMessage(Long_t msg, Long_t param1, Long_t param2);
   virtual void UpdateParamFile();

private:
   TGComboBox       * m_comboBox;
   TGCompositeFrame * m_win;
};

/////////////////////////////////////////////////////////////////////

class W_IntEntry : public Widget
{
public:
   W_IntEntry(const char * name); 

   virtual void Create(TGCompositeFrame * win, const char * winName);
   virtual void ProcessMessage(Long_t msg, Long_t param1, Long_t param2);
   virtual void ValueChanged(string & widgetName, string & value);
   virtual void UpdateParamFile();

private:
   TGNumberEntry * m_entry;

};

/////////////////////////////////////////////////////////////////////

class W_RealEntry : public Widget
{
public:
   W_RealEntry(const char * name);

   virtual void Create(TGCompositeFrame * win, const char * winName);
   virtual void ProcessMessage(Long_t msg, Long_t param1, Long_t param2);
   virtual void ValueChanged(string & widgetName, string & value);
   virtual void UpdateParamFile();

private:
   TGNumberEntry * m_entry;

};

/////////////////////////////////////////////////////////////////////

class W_TextEntry : public Widget
{
public:
   W_TextEntry(const char * name, int type);

   virtual void Create(TGCompositeFrame * win, const char * winName);

   virtual void ProcessMessage(Long_t msg, Long_t param1, Long_t param2);
   virtual void ValueChanged(string & widgetName, string & value);
   virtual void UpdateParamFile();

private:
   Int_t         m_textId;

   TGTextEntry   * m_entry;
   TGTextBuffer  * m_buffer;
   TGTextButton  * m_button;
};

/////////////////////////////////////////////////////////////////////


#endif // WIDGET_INCLUDED







