#include <stdio.h>

#include <TFile.h>
#include <TGraphErrors.h>

#include "pil.h"
#include "param_api.h"

#include "TFError.h"
#include "TFTable.h"
#include "TFGroup.h"
#include "TFImage.h"

struct Params
{
   char inFile[PIL_LINESIZE];
   int  allData;
   char elementName[PIL_LINESIZE];
   int  cycle;

   char outFile[PIL_LINESIZE];

   int  table2tree;
   char convert[PIL_LINESIZE];
   char conv_to_[PIL_LINESIZE];

   int  table2graph;
   char x_column[PIL_LINESIZE];
   char y_column[PIL_LINESIZE];
   char x_err_col[PIL_LINESIZE];
   char y_err_col[PIL_LINESIZE];

   int  image2histogram;
   TClass * histoType;

   int  image2tree;
};


//_____________________________________________________________________________

class NameConvert : public TFNameConvert
{
   int   m_to_lower_upper;  // 0: no conversino, 1: to lower, 2: to upper
   char  m_to_[40];
public:
   NameConvert(const char * lower_upper, const char * to_);

   virtual const char * Conv (const char * name);
   virtual const char * ConvHeader (const char * name);
};

NameConvert::NameConvert(const char * lower_upper, const char * to_)
{
   if (strcmp(lower_upper, "TO_LOWER") == 0)
      m_to_lower_upper = 1;
   else if (strcmp(lower_upper, "TO_UPPER") == 0)
      m_to_lower_upper = 2;
   else
      m_to_lower_upper = 0;

   strcpy(m_to_, to_);
}

const char * NameConvert::Conv(const char * name)
{
   static char conName[100];
   
   int index = 0;      
   while (index < 100)
      {
      if (m_to_lower_upper == 1)
         conName[index] = tolower(name[index]);
      else if (m_to_lower_upper == 2)
         conName[index] = toupper(name[index]);
      else
         conName[index] = name[index];
   
      if (name[index] == 0)
         break;
   
      if (strchr(m_to_, conName[index]) ) 
        conName[index] = '_';


      index++;
      }

   conName[99] = 0;
   return conName;
}

const char * NameConvert::ConvHeader(const char * name)
{
   static char conName[100];
   strcpy(conName, Conv(name));
   strcat(conName, "_h");
   return conName;
}

//_____________________________________________________________________________

static void Convert(Params & params, TFIOElement * element)
{
   if ( element == NULL ) return;

   Bool_t      elementConverted = kFALSE;

   if ( params.table2tree && element->IsA()->InheritsFrom(TFTable::Class()) ) 
      {
      ((TFTable*)element)->MakeTree(new NameConvert(params.convert, params.conv_to_));
      elementConverted = kTRUE;
      }

   if (params.table2graph && element->IsA()->InheritsFrom(TFTable::Class()) ) 
      {
      TGraphErrors * graph = ((TFTable*)element)->MakeGraph(
                             params.x_column, params.y_column,
                             params.x_err_col, params.y_err_col);
 
      NameConvert nc(params.convert, params.conv_to_);
      graph->Write(nc.Conv(element->GetName()));
      elementConverted = kTRUE;
      }
      
   if (params.image2histogram && element->IsA()->InheritsFrom(TFBaseImage::Class()) ) 
      {
      UInt_t numDim = ((TFBaseImage*)element)->GetNumDim();
      if (numDim > 3)
         return;

      UInt_t size[3] = {1, 0, 0};
      if (numDim == 3)
          ((TFBaseImage*)element)->GetSize(size);

      if (size[0] > 20) 
         size[0] = 20; // if it is greater than 20 it properly does not make
                       // sence to build one hitorgram per slice.

      NameConvert nc(params.convert, params.conv_to_);
      for (UInt_t zPos = 0; zPos < size[0]; zPos++)
          {
         TH1 * hist = ((TFBaseImage*)element)->MakeHisto(zPos, params.histoType);

         hist->Write(nc.Conv(element->GetName()));
         elementConverted = kTRUE;
         delete hist;
         }
      }

   if (params.image2tree && element->IsA()->InheritsFrom(TFBaseImage::Class()) ) 
      {
      ((TFBaseImage*)element)->MakeTree(new NameConvert(params.convert, params.conv_to_));
      elementConverted = kTRUE;
      }
        
         
   if (elementConverted) 
      {
      NameConvert nc(params.convert, params.conv_to_);

      TFIOElement *head = new TFIOElement(*element);
      head->SetTitle("header");
      head->Write(nc.ConvHeader(element->GetName()));
      delete head;
      }

}

//_____________________________________________________________________________

static int ReadParams(Params * params, int argc, char ** argv)
{
   int status = ISDC_OK;

   for (int par = 1; par < argc; par++)
      if (strcmp("--g",   argv[par]) == 0 ||
          strcmp("--gui", argv[par]) == 0    )
         {
         // use the GUI to ask for the parameters
         ParamGui gui(argc, argv);
         if (gui.Quit())        exit(0);
         if (gui.Error() != ISDC_OK)   return gui.Error();
      
         strcpy(params->inFile,      gui.GetString("inFile"));
         params->allData =           gui.GetBool("all_data");
         strcpy(params->elementName, gui.GetString("element_name"));
         params->cycle =             gui.GetInt("cycle");

         strcpy(params->outFile,     gui.GetString("outFile"));

         params->table2tree =        gui.GetBool("table_to_tree");
         strcpy(params->convert,     gui.GetString("name_convert"));
         strcpy(params->conv_to_,    gui.GetString("convert_to_underline"));

         params->table2graph =       gui.GetBool("table_to_graph");
         strcpy(params->x_column,    gui.GetString("x_column"));
         strcpy(params->y_column,    gui.GetString("y_column"));
         strcpy(params->x_err_col,   gui.GetString("x_err_col"));         strcpy(params->y_err_col,   gui.GetString("y_err_col"));


         params->image2histogram =   gui.GetBool("image_to_histogram");
         char histoTypeStr[10];
         strcpy(histoTypeStr,        gui.GetString("histogram_type"));
         if ( strcmp(histoTypeStr, "TH1C" ) == 0 ) params->histoType = TH1C::Class();
         else if ( strcmp(histoTypeStr, "TH1S" ) == 0 ) params->histoType = TH1S::Class();
         else if ( strcmp(histoTypeStr, "TH1F" ) == 0 ) params->histoType = TH1F::Class();
         else if ( strcmp(histoTypeStr, "TH1D" ) == 0 ) params->histoType = TH1D::Class();
         else if ( strcmp(histoTypeStr, "TH2C" ) == 0 ) params->histoType = TH2C::Class();
         else if ( strcmp(histoTypeStr, "TH2S" ) == 0 ) params->histoType = TH2S::Class();
         else if ( strcmp(histoTypeStr, "TH2F" ) == 0 ) params->histoType = TH2F::Class();
         else if ( strcmp(histoTypeStr, "TH2D" ) == 0 ) params->histoType = TH2D::Class();

         params->image2tree =   gui.GetBool("image_to_tree");


         return gui.Error();
         }

   do
      {
      status = PILInit(argc, argv);
      if (status != ISDC_OK) break;

      status = PILGetFname("inFile", params->inFile);
      if (status != ISDC_OK) break;

      status = PILGetFname("outFile", params->outFile);
      if (status != ISDC_OK) break;

      status = PILGetBool("all_data", &params->allData);
      if (status != ISDC_OK) break;

      if (!params->allData)
         {
         status = PILGetString("element_name", params->elementName);
         if (status != ISDC_OK) break;
         
         status = PILGetInt("cycle", &params->cycle);
         if (status != ISDC_OK) break;
         }

      status = PILGetBool("table_to_tree", &params->table2tree);
      if (status != ISDC_OK) break;

      if (params->table2tree) 
         {
         status = PILGetString("name_convert", params->convert);
         if (status != ISDC_OK) break;

         status = PILGetString("convert_to_underline", params->conv_to_);
         if (status != ISDC_OK) break;
         }


      status = PILGetBool("table_to_graph", &params->table2graph);
      if (status != ISDC_OK) break;

      if (params->table2graph) 
         {
         status = PILGetString("x_column", params->x_column);
         if (status != ISDC_OK) break;

         status = PILGetString("y_column", params->y_column);
         if (status != ISDC_OK) break;

         status = PILGetString("x_err_col", params->x_err_col);
         if (status != ISDC_OK) break;

         status = PILGetString("y_err_col", params->y_err_col);
         if (status != ISDC_OK) break;

         }


      status = PILGetBool("image_to_histogram", &params->image2histogram);
      if (status != ISDC_OK) break;

      if (params->image2histogram) 
         {
         char histoTypeStr[10];
         status = PILGetString("histogram_type", histoTypeStr);
         if (status != ISDC_OK) break;

         if ( strcmp(histoTypeStr, "TH1C" ) == 0 ) params->histoType = TH1C::Class();
         else if ( strcmp(histoTypeStr, "TH1S" ) == 0 ) params->histoType = TH1S::Class();
         else if ( strcmp(histoTypeStr, "TH1F" ) == 0 ) params->histoType = TH1F::Class();
         else if ( strcmp(histoTypeStr, "TH1D" ) == 0 ) params->histoType = TH1D::Class();
         else if ( strcmp(histoTypeStr, "TH2C" ) == 0 ) params->histoType = TH2C::Class();
         else if ( strcmp(histoTypeStr, "TH2S" ) == 0 ) params->histoType = TH2S::Class();
         else if ( strcmp(histoTypeStr, "TH2F" ) == 0 ) params->histoType = TH2F::Class();
         else if ( strcmp(histoTypeStr, "TH2D" ) == 0 ) params->histoType = TH2D::Class();

         }

      status = PILGetBool("image_to_tree", &params->image2tree);
      if (status != ISDC_OK) break;
   

      } while (0);

   PILClose(ISDC_OK);

   return status;
}
//_____________________________________________________________________________
main(int argc, char * argv[])
{

   Params params;
   int status = ReadParams(&params, argc, argv);
   if (status != ISDC_OK)
      {
      printf("Error during reading parameters: %d\n", status);
      exit(1);
      }

   // open a ROOT file to store the TTrees we will create
   TFile * fl = new TFile(params.outFile, "recreate");

   if (params.allData)
      {
      // all tables of a file
      TFFileIter i_file(params.inFile);
      while (i_file.Next())
         Convert(params, &*i_file);
      }
   else
      {
      // open one element of the input file
      TFIOElement * element = TFRead(params.inFile, params.elementName, params.cycle);
      Convert(params, element);
      delete element;
      }
   
   // write all created TTrees into the ROOT file and close it
   fl->Write();
   delete fl;

   // print the error if there was one
   TFError::PrintErrors();
}
