// ///////////////////////////////////////////////////////////////////
//
//  File:      tree2tf.cxx  
//
//  Version:   1.0
//
//  Author:    Reiner Rohlfs
//
//  History:
//
// ///////////////////////////////////////////////////////////////////
#include <stdio.h>

#include <TFile.h>
#include <TKey.h>
#include <TList.h>
#include <TGraphErrors.h>

#include "pil.h"
#include "param_api.h"

#include "TFError.h"
#include "TFTable.h"
#include "TFGroup.h"
#include "TFImage.h"

struct Params
{
   char inFile[PIL_LINESIZE];
   int  allData;
   char treeName[100];

   char outFile[PIL_LINESIZE];

};

//_____________________________________________________________________________
static void Convert(TTree * tree, Params & params)
{
// does the work of this program, creates a TFTable from the tree
// and saves it.

   TFTable * table = new TFTable(tree);
   table->SaveElement(params.outFile);
   delete table;
}

//_____________________________________________________________________________
static int ReadParams(Params * params, int argc, char ** argv)
{
   int status = ISDC_OK;

   for (int par = 1; par < argc; par++)
      if (strcmp("--g",   argv[par]) == 0 ||
          strcmp("--gui", argv[par]) == 0    )
         {
         // use the GUI to ask for the parameters
         ParamGui gui(argc, argv);
         if (gui.Quit())        exit(0);
         if (gui.Error() != ISDC_OK)   return gui.Error();
      
         strcpy(params->inFile,      gui.GetString("inFile"));
         params->allData =           gui.GetBool("all_data");
         strcpy(params->treeName,    gui.GetString("tree_name"));

         strcpy(params->outFile,     gui.GetString("outFile"));

         return gui.Error();
         }

   do
      {
      status = PILInit(argc, argv);
      if (status != ISDC_OK) break;

      status = PILGetFname("inFile", params->inFile);
      if (status != ISDC_OK) break;

      status = PILGetBool("all_data", &params->allData);
      if (status != ISDC_OK) break;

      if (!params->allData)
         {
         status = PILGetString("tree_name", params->treeName);
         if (status != ISDC_OK) break;
         }

      status = PILGetFname("outFile", params->outFile);
      if (status != ISDC_OK) break;

      } while (0);

   PILClose(ISDC_OK);

   return status;
}
//_____________________________________________________________________________
main(int argc, char * argv[])
{

   Params params;
   int status = ReadParams(&params, argc, argv);
   if (status != ISDC_OK)
      {
      printf("Error during reading parameters: %d\n", status);
      exit(1);
      }

   // open a ROOT file to store the TTrees we will create
   TFile * fl = new TFile(params.inFile, "read");

   if (params.allData)
      {
      // process all tress in the input ROOT file
      TIter next(fl->GetListOfKeys());
      while (TKey * key = (TKey*)next())
         {
         TTree * tree = (TTree*)fl->Get(key->GetName());
         if (tree && tree->IsA()->InheritsFrom(TTree::Class()) )
            Convert(tree, params);
         }
      }
   else
      {
      // process one tree in the file definede by its name
      TTree * tree = (TTree*)fl->Get(params.treeName);
      if (tree && tree->IsA()->InheritsFrom(TTree::Class()) )
         {
         Convert(tree, params);
         }

      }


   delete fl;

   // print the error if there was one
   TFError::PrintErrors();
}
